package org.jboss.soa.esb.samples.quickstart.juddi;

import org.apache.commons.configuration.ConfigurationException;

import org.apache.juddi.v3.client.config.UDDIClerkManager;
import org.apache.juddi.v3.client.config.UDDIClientContainer;
import org.apache.juddi.v3.client.transport.Transport;
import org.apache.juddi.v3.client.transport.TransportException;

import org.apache.juddi.v3_service.JUDDIApiPortType;

import org.apache.log4j.Logger;

import org.uddi.v3_service.UDDICustodyTransferPortType;
import org.uddi.v3_service.UDDIInquiryPortType;
import org.uddi.v3_service.UDDIPublicationPortType;
import org.uddi.v3_service.UDDISecurityPortType;
import org.uddi.v3_service.UDDISubscriptionListenerPortType;
import org.uddi.v3_service.UDDISubscriptionPortType;

public class JUddiClient {

	public static String DEFAULT_NODE_NAME = "default";
	
	private static Logger logger = Logger.getLogger(JUddiClient.class);
	
	private Transport transport;
	
	public JUddiClient(String configFile) {
		this(configFile, DEFAULT_NODE_NAME);
	}
	
	public JUddiClient(String configFile, String nodeName) {
		try {
			init(new UDDIClerkManager(configFile), nodeName);
		} catch (ConfigurationException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e);
		}
	}
	
	public JUddiClient(UDDIClerkManager manager, String nodeName) {
		init(manager, nodeName);
	}
	
	private void init(UDDIClerkManager manager, String nodeName) throws RuntimeException {
		try {
			manager.start();
			
			UDDIClientContainer.addClerkManager(manager);

			String clazz = manager.getClientConfig().getUDDINode(nodeName).getProxyTransport();
			Class<?> transportClass = Utils.forName(clazz, Transport.class);
			
			transport = (Transport) transportClass.getConstructor(String.class).newInstance(nodeName);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e);
		}
	}
	
	public UDDIPublicationPortType getPublicationPort() throws TransportException {
		try {
			return transport.getUDDIPublishService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public UDDIInquiryPortType getInquiryPort() throws TransportException {
		try {
			return transport.getUDDIInquiryService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public UDDISecurityPortType getSecurityPort() throws TransportException {
		try {
			return transport.getUDDISecurityService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public UDDICustodyTransferPortType getCustodyTransferPort() throws TransportException {
		try {
			return transport.getUDDICustodyTransferService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public UDDISubscriptionPortType getSubscriptionPort() throws TransportException {
		try {
			return transport.getUDDISubscriptionService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public UDDISubscriptionListenerPortType getSubscriptionListenerPort() throws TransportException {
		try {
			return transport.getUDDISubscriptionListenerService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public JUDDIApiPortType getApiPort() throws TransportException {
		try {
			return transport.getJUDDIApiService();
		} catch (TransportException e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

}
