package org.jboss.soa.esb.samples.quickstart.juddi;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.juddi.api_v3.SyncSubscription;
import org.apache.juddi.api_v3.SyncSubscriptionDetail;

import org.apache.juddi.v3.client.config.ClientConfig;
import org.apache.juddi.v3.client.config.UDDIClerk;
import org.apache.juddi.v3_service.JUDDIApiPortType;

import org.jboss.soa.esb.samples.quickstart.juddi.tck.TckBusinessService;

import org.testng.annotations.Test;

import org.uddi.sub_v3.CoveragePeriod;
import org.uddi.sub_v3.GetSubscriptionResults;
import org.uddi.sub_v3.SubscriptionResultsList;



@Test(groups = { "juddiv3", "clustered" }, enabled = true)
public class JUddiFederatedRegistryTest extends SubscriptionBaseTest {
	
	public static final String SUBSCRIPTION_URL_1 = "http://127.0.0.1:8080/juddiv3/services/subscription"; 
	public static final String SUBSCRIPTION_URL_2 = "http://127.0.0.1:8180/juddiv3/services/subscription";
	
	public static final String SALES_TMODEL = "juddi_data/xml/sales/sales_tModels.xml";
	public static final String SALES_MARKETING_TMODEL = "juddi_data/xml/sales/marketing_tModels.xml";
	public static final String SALES_BUSINESS = "juddi_data/xml/sales/sales_BusinessEntity.xml";
	public static final String SALES_BUSINESS_SERVICE = "juddi_data/xml/sales/new_businessService.xml";
	
	public static final String MARKETING_TMODEL = "juddi_data/xml/marketing/marketing_tModels.xml";
	public static final String MARKETING_SALES_TMODEL = "juddi_data/xml/marketing/sales_tModels.xml";
	public static final String MARKETING_BUSINESS = "juddi_data/xml/marketing/marketing_BusinessEntity.xml";
	public static final String MARKETING_BUSINESS_SERVICE = "juddi_data/xml/marketing/new_businessService.xml";
	
	public static final String SALES_SOAP_SUBSCRIPTION = "juddi_data/soap/sales/save_subscription.xml";
	public static final String MARKETING_SOAP_SUBSCRIPTION = "juddi_data/soap/marketing/save_subscription.xml";
	
	private TckBusinessService tckBusinessService1;
	private TckBusinessService tckBusinessService2;
	
	public JUddiFederatedRegistryTest() {
		super();
		tckBusinessService1 = new TckBusinessService(connector1.getPublicationPort(), connector1.getInquiryPort());
		tckBusinessService2 = new TckBusinessService(connector2.getPublicationPort(), connector2.getInquiryPort());
	}
	
	// Save info to the target identified by the toClerk (localhost)
	protected void saveSubscription1() throws Exception {
		try {
			ClientConfig cfg = getSalesClerkManager().getClientConfig();
			
			JUDDIApiPortType salesApiPort = connector1.getApiPort();
			
			UDDIClerk fromClerk = cfg.getUDDIClerks().get("RemotehostCratchit"); // remote
			UDDIClerk toClerk   = cfg.getUDDIClerks().get("LocalhostCratchit");  // local
			
			saveNode(fromClerk.getApiClerk().getNode(), salesLocalAuthInfo, salesApiPort);
			saveNode(toClerk.getApiClerk().getNode(), salesLocalAuthInfo, salesApiPort);
			
			saveClerk(fromClerk.getApiClerk(), salesLocalAuthInfo, salesApiPort);
			saveClerk(toClerk.getApiClerk(), salesLocalAuthInfo, salesApiPort);
			
			saveClientSubscription(salesLocalAuthInfo, fromClerk.getApiClerk(), toClerk.getApiClerk(), "uddi:sales.apache.org:subscription:key1", salesApiPort);
			saveSubscription(SUBSCRIPTION_URL_2, SALES_SOAP_SUBSCRIPTION, salesRemoteAuthInfo);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	protected void saveSubscription2() throws Exception {
		try {
			ClientConfig cfg = getMarketingClerkManager().getClientConfig();
			
			JUDDIApiPortType salesApiPort = connector2.getApiPort();
			
			UDDIClerk fromClerk = cfg.getUDDIClerks().get("RemotehostCratchit"); // save remote
			UDDIClerk toClerk   = cfg.getUDDIClerks().get("LocalhostCratchit");  // save local
			
			saveNode(fromClerk.getApiClerk().getNode(), marketingLocalAuthInfo, salesApiPort);
			saveNode(toClerk.getApiClerk().getNode(), marketingLocalAuthInfo, salesApiPort);
			
			saveClerk(fromClerk.getApiClerk(), marketingLocalAuthInfo, salesApiPort);
			saveClerk(toClerk.getApiClerk(), marketingLocalAuthInfo, salesApiPort);
			
			saveClientSubscription(marketingLocalAuthInfo, fromClerk.getApiClerk(), toClerk.getApiClerk(), "uddi:marketing.apache.org:subscription:key1", salesApiPort);
			saveSubscription(SUBSCRIPTION_URL_1, MARKETING_SOAP_SUBSCRIPTION, marketingRemoteAuthInfo);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	// Invoke sync on localhost
	protected SubscriptionResultsList invokeSyncSubscription1() throws Exception {
		try {
			JUDDIApiPortType apiPort = connector1.getApiPort();
			
			GetSubscriptionResults getSubscriptionResults = new GetSubscriptionResults();
			// this will result to a remote call
			getSubscriptionResults.setAuthInfo(salesRemoteAuthInfo);
			getSubscriptionResults.setSubscriptionKey("uddi:sales.apache.org:subscription:key1");
			
			XMLGregorianCalendar calendarStart = DatatypeFactory.newInstance().newXMLGregorianCalendar("2010-01-01T00:00:00");
			XMLGregorianCalendar calendarEnd = DatatypeFactory.newInstance().newXMLGregorianCalendar("2012-01-01T00:00:00");
			CoveragePeriod coverage = new CoveragePeriod();
			coverage.setStartPoint(calendarStart);
			coverage.setEndPoint(calendarEnd);
			getSubscriptionResults.setCoveragePeriod(coverage);
			
			SyncSubscription syncSubscription = new SyncSubscription();
	        syncSubscription.getGetSubscriptionResultsList().add(getSubscriptionResults);
	        syncSubscription.setAuthInfo(salesLocalAuthInfo);
	        // This will result to a local call. The remote will come when the api loads the 
	        // the fromClerk configuration and starts to process GetSubscriptionResults on the remote server
	        SyncSubscriptionDetail detail = apiPort.invokeSyncSubscription(syncSubscription);
			SubscriptionResultsList list = detail.getSubscriptionResultsList().get(0);
			
			return list;
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	protected SubscriptionResultsList invokeSyncSubscription2() throws Exception {
		try {
			JUDDIApiPortType apiPort = connector2.getApiPort();
			
			GetSubscriptionResults getSubscriptionResults = new GetSubscriptionResults();
			getSubscriptionResults.setAuthInfo(marketingRemoteAuthInfo);
			getSubscriptionResults.setSubscriptionKey("uddi:marketing.apache.org:subscription:key1");
			
			XMLGregorianCalendar calendarStart = DatatypeFactory.newInstance().newXMLGregorianCalendar("2010-01-01T00:00:00");
			XMLGregorianCalendar calendarEnd = DatatypeFactory.newInstance().newXMLGregorianCalendar("2012-01-01T00:00:00");
			CoveragePeriod coverage = new CoveragePeriod();
			coverage.setStartPoint(calendarStart);
			coverage.setEndPoint(calendarEnd);
			getSubscriptionResults.setCoveragePeriod(coverage);
	        
			SyncSubscription syncSubscription = new SyncSubscription();
	        syncSubscription.setAuthInfo(marketingLocalAuthInfo);
	        syncSubscription.getGetSubscriptionResultsList().add(getSubscriptionResults);
	        
	        SyncSubscriptionDetail detail = apiPort.invokeSyncSubscription(syncSubscription);
			SubscriptionResultsList list = detail.getSubscriptionResultsList().get(0);
			
			return list;
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	@Test (enabled = true)
	public void testFedaratedRegistries() throws Exception {
		// save nodes, clerks (listeners)
		saveSubscription1();
		saveSubscription2();
		
		// do some changes on server 1
		tckBusinessService1.saveServices(salesLocalAuthInfo, SALES_BUSINESS_SERVICE, "uddi:sales.apache.org:new-service", 2);
		tckBusinessService2.saveServices(marketingLocalAuthInfo, MARKETING_BUSINESS_SERVICE, "uddi:marketing.apache.org:new-service", 2);
		
		// invoke sync
		invokeSyncSubscription1();
		invokeSyncSubscription2();
		
		// assert
		// TODO
	}
	
}