package org.jboss.soa.esb.samples.quickstart.juddi;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.juddi.v3_service.JUDDIApiPortType;

import org.uddi.v3_service.UDDICustodyTransferPortType;
import org.uddi.v3_service.UDDIInquiryPortType;
import org.uddi.v3_service.UDDIPublicationPortType;
import org.uddi.v3_service.UDDIReplicationPortType;
import org.uddi.v3_service.UDDISecurityPortType;
import org.uddi.v3_service.UDDISubscriptionListenerPortType;
import org.uddi.v3_service.UDDISubscriptionPortType;

public class JUddiRmiConnector {

	static final String DEFAULT_NAMING_PROVIDER = "jnp://localhost:1099";
	static final String NAMING_PROVIDER         = "java.naming.provider.url";
	static final String JNDI_ROOT 			    = "juddiv3";
	
	private InitialContext context				= null;
	
	private UDDIPublicationPortType          publicationPort		  = null;
	private UDDIInquiryPortType              inquiryPort 			  = null;
	private UDDISecurityPortType             securityPort			  = null;
	private UDDICustodyTransferPortType      custodyTransferPort	  = null;
	private UDDISubscriptionPortType         subscriptionPort		  = null;
	private UDDISubscriptionListenerPortType subscriptionListenerPort = null;
	private JUDDIApiPortType                 apiPort				  = null;
	private UDDIReplicationPortType          replicationPort		  = null;
	
	public JUddiRmiConnector(Class<?> clazz) {
		this(clazz, System.getProperty(NAMING_PROVIDER, DEFAULT_NAMING_PROVIDER));
	}
	
	/**
	 * TODO: fix Exception handling (null when properties are not loaded)
	 * 
	 * @param clazz
	 * @param providerUrl
	 */
	public JUddiRmiConnector(Class<?> clazz, String providerUrl) {
		String masterURL = System.getProperty(NAMING_PROVIDER, DEFAULT_NAMING_PROVIDER);
		try {
			System.setProperty(NAMING_PROVIDER, providerUrl);
			context = new InitialContext();
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
		System.setProperty(NAMING_PROVIDER, masterURL);
	}
	
	public UDDISecurityPortType getSecurityPort() {
		try {
			if(securityPort == null) {
				securityPort = (UDDISecurityPortType) 
					context.lookup(JNDI_ROOT + "/UDDISecurityService");
			}
			return securityPort;
		
		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public UDDIInquiryPortType getInquiryPort() {
		try {
			if(inquiryPort == null) {
				inquiryPort = (UDDIInquiryPortType) 
					context.lookup(JNDI_ROOT + "/UDDIInquiryService");
			}
			return inquiryPort;

		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public UDDIPublicationPortType getPublicationPort() {
		try {
			if(publicationPort == null) {
				publicationPort = (UDDIPublicationPortType) 
					context.lookup(JNDI_ROOT + "/UDDIPublicationService");
			}
			return publicationPort;
			
		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public UDDICustodyTransferPortType getCustodyTransferPort() {
		try {
			if(custodyTransferPort == null) {
				custodyTransferPort = (UDDICustodyTransferPortType) 
					context.lookup(JNDI_ROOT + "/UDDICustodyTransferService");
			}
			return custodyTransferPort;
			
		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public UDDISubscriptionPortType getSubscriptionPort() {
		try {
			if(subscriptionPort == null) {
				subscriptionPort = (UDDISubscriptionPortType) 
					context.lookup(JNDI_ROOT + "/UDDISubscriptionService");
			}
			return subscriptionPort;
			
		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public UDDISubscriptionListenerPortType getSubscriptionListenerPort() {
		try {
			if(subscriptionListenerPort == null) {
				subscriptionListenerPort = (UDDISubscriptionListenerPortType) 
					context.lookup(JNDI_ROOT + "/UDDISubscriptionListenerService");
			}
			return subscriptionListenerPort;
			
		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public JUDDIApiPortType getApiPort() {
		try {
			if(apiPort == null) {
				apiPort = (JUDDIApiPortType) context.lookup(JNDI_ROOT + "/JUDDIApiService");
			}
			return apiPort;
			
		} catch (NamingException e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e.getMessage(), e.getCause());
		}
	}
	
	public UDDIReplicationPortType getReplicationPort() {
//		try {
//			if(replicationPort == null) {
//				replicationPort = (UDDIReplicationPortType) context.lookup(JNDI_ROOT + "/UDDIReplicationService");
//			}
//			return replicationPort;
//			
//		} catch (NamingException ex) {
//			throw new TestRunException(ex.getMessage(), ex.getCause());
//		}
		throw new UnsupportedOperationException("Not implemented yet.");
	}
	
	
}
