package org.jboss.soa.esb.samples.quickstart.juddi;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.juddi.api_v3.Clerk;
import org.apache.juddi.api_v3.ClientSubscriptionInfo;
import org.apache.juddi.api_v3.DeleteClientSubscriptionInfo;
import org.apache.juddi.api_v3.Node;
import org.apache.juddi.api_v3.SaveClerk;
import org.apache.juddi.api_v3.SaveClientSubscriptionInfo;
import org.apache.juddi.api_v3.SaveNode;

import org.apache.juddi.v3.client.config.UDDIClerk;
import org.apache.juddi.v3.client.config.UDDIClerkManager;
import org.apache.juddi.v3.client.config.UDDINode;
import org.apache.juddi.v3_service.JUDDIApiPortType;

import org.jboss.soa.esb.samples.quickstart.juddi.soap.JUddiSoapUtils;
import org.jboss.soa.esb.samples.quickstart.juddi.tck.TckResourceLoader;
import org.jboss.soa.esb.samples.quickstart.juddi.tck.TckSecurity;


public class SubscriptionBaseTest extends JUddiRmiClusteredBaseTest {

	protected String salesLocalAuthInfo;
	protected String salesRemoteAuthInfo;
	
	protected String marketingLocalAuthInfo;
	protected String marketingRemoteAuthInfo;
	
	private List<UDDIClerkManager> clerkManagerList;
	
	public SubscriptionBaseTest() {
		super();
		clerkManagerList = new ArrayList<UDDIClerkManager>();
		// should get through at least for i=1
		try {
			for(int i=1;; i++) {
				String configPath = System.getProperty("uddi.client.xml." + i);
				
				if(configPath == null) {
					break;
				} else {
					System.out.println("#### ConfigPath " + i + ": " + configPath);
				}
				
				UDDIClerkManager manager = new UDDIClerkManager(configPath);
				manager.start();
				clerkManagerList.add(manager);
			}
			
			salesLocalAuthInfo = TckSecurity.getAuthToken(connector1.getSecurityPort(), "sales", "");
			salesRemoteAuthInfo = TckSecurity.getAuthToken(clerkManagerList.get(0), "RemotehostCratchit");
			marketingLocalAuthInfo = TckSecurity.getAuthToken(connector2.getSecurityPort(), "marketing", "");
			marketingRemoteAuthInfo = TckSecurity.getAuthToken(clerkManagerList.get(1), "RemotehostCratchit");
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw new RuntimeException(e);
		}
	}
	
	protected UDDIClerkManager getSalesClerkManager() {
		return clerkManagerList.get(0);
	}
	
	protected UDDIClerkManager getMarketingClerkManager() {
		return clerkManagerList.get(1);
	}
	
	protected void saveClientSubscription(String authInfo, Clerk fromClerk, Clerk toClerk, String subscriptionKey, JUDDIApiPortType apiPort) 
		throws Exception 
	{
		try {
			ClientSubscriptionInfo clientSubscriptionInfo = new ClientSubscriptionInfo();
			clientSubscriptionInfo.setFromClerk(fromClerk);
			clientSubscriptionInfo.setToClerk(toClerk);
			clientSubscriptionInfo.setSubscriptionKey(subscriptionKey);
			clientSubscriptionInfo.setLastModified(new Date());
	
			SaveClientSubscriptionInfo saveClientSubscriptionInfo = new SaveClientSubscriptionInfo();
			saveClientSubscriptionInfo.getClientSubscriptionInfo().add(
					clientSubscriptionInfo);
			saveClientSubscriptionInfo.setAuthInfo(authInfo);

			apiPort.saveClientSubscriptionInfo(saveClientSubscriptionInfo);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	protected void deleteClientSubscription(String authInfo, String subscriptionKey, JUDDIApiPortType apiPort) 
		throws Exception 
	{
		try {
			DeleteClientSubscriptionInfo dsi = new DeleteClientSubscriptionInfo();
			dsi.setAuthInfo(authInfo);
			dsi.getSubscriptionKey().add(subscriptionKey);
			
			apiPort.deleteClientSubscriptionInfo(dsi);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	protected void saveClerk(Clerk clerk, String authInfo, JUDDIApiPortType apiPort) 
		throws Exception 
	{
		try {
			SaveClerk sClerk = new SaveClerk();
			sClerk.setAuthInfo(authInfo);
			sClerk.getClerk().add(clerk);
			apiPort.saveClerk(sClerk);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	protected void saveSubscription(String url, String subscriptionSoap, String authInfo) throws Exception {
		try {
			String soapMessageNoAuth = TckResourceLoader.loadFileContent(subscriptionSoap);
			JUddiSoapUtils.sendJUddiSoapMessage(url, soapMessageNoAuth, "save_subscription", authInfo);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	protected void deleteSubscription() {
		// TODO
	}
	
	// We must use JAXWS until jUDDI RMI api is fixed
	protected void saveNode(Node node, String authInfo, JUDDIApiPortType apiPort)
			throws Exception 
	{
		try {
			SaveNode sNode = new SaveNode();
			sNode.setAuthInfo(authInfo);
			sNode.getNode().add(node);
			apiPort.saveNode(sNode);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	protected Node mapUDDINodeToNode(UDDINode uddiNode) {
		Node node = new Node();
		node.setName(uddiNode.getName());
		node.setDescription(uddiNode.getDescription());
		node.setManagerName(uddiNode.getManagerName());

		node.setCustodyTransferUrl(uddiNode.getCustodyTransferUrl());
		node.setFactoryInitial(uddiNode.getFactoryInitial());
		node.setFactoryNamingProvider(uddiNode.getFactoryNamingProvider());
		node.setFactoryURLPkgs(uddiNode.getFactoryURLPkgs());
		node.setInquiryUrl(uddiNode.getInquiryUrl());
		node.setJuddiApiUrl(uddiNode.getJuddiApiUrl());
		node.setProxyTransport(uddiNode.getProxyTransport());
		node.setPublishUrl(uddiNode.getPublishUrl());
		node.setSecurityUrl(uddiNode.getSecurityUrl());
		node.setSubscriptionListenerUrl(uddiNode.getSubscriptionListenerUrl());
		node.setSubscriptionUrl(uddiNode.getSubscriptionUrl());

		return node;
	}

	protected Clerk mapUDDIClerkToClerk(UDDIClerk uddiClerk) {
		Clerk clerk = new Clerk();
		clerk.setName(uddiClerk.getName());
		clerk.setNode(mapUDDINodeToNode(uddiClerk.getUDDINode()));
		clerk.setPublisher(uddiClerk.getPublisher());
		clerk.setPassword(uddiClerk.getPassword());

		return clerk;
	}

}
