package org.jboss.soa.esb.samples.quickstart.juddi.soap;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSSerializer;
import org.xml.sax.InputSource;

public class JUddiSoapUtils {

	public static String sendSoapMessage(String url, String soapMessage, String soapAction) throws Exception 
	{
		try {
			URL endpoint = new URL(url);
			HttpURLConnection httpConnection = (HttpURLConnection) endpoint
					.openConnection();
	
			httpConnection.setDoInput(true);
			httpConnection.setDoOutput(true);
			httpConnection.setUseCaches(false);
			httpConnection.setRequestMethod("POST");
			httpConnection.setRequestProperty("Content-Type", "text/xml");
			httpConnection.setRequestProperty("SOAPAction", soapAction);
			httpConnection.connect();
	
			PrintWriter pw = new PrintWriter(httpConnection.getOutputStream());
			pw.write(soapMessage);
			pw.flush();
			pw.close();
	
			BufferedReader in = new BufferedReader(new InputStreamReader(
					httpConnection.getInputStream()));
			String response = readReader(in);
			in.close();
	
			return response;
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	public static void sendJUddiSoapMessage(String url, String soapMessageNoAuth, String soapAction, String authInfo) throws Exception 
	{
		try {
			String message = setAuthInfoElement(authInfo, soapMessageNoAuth);
			sendSoapMessage(url, message, soapAction);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	public static String getAuthInfo(String securityURL, String soapResource)
			throws Exception 
	{
		try {
			String soapRequest = readTestAsciiResource(soapResource);
			String soapResponse = sendSoapMessage(securityURL, soapRequest,
					"get_authToken");
	
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
	
			Document doc = builder.parse(new InputSource(new StringReader(
					soapResponse)));
			Node node = doc.getElementsByTagName("authInfo").item(0);
			return node.getTextContent();
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	public static String setAuthInfoElement(String authInfo, String uddiSoapMessage)
			throws Exception 
	{
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
	
			InputSource src = new InputSource(new StringReader(uddiSoapMessage));
			Document doc = builder.parse(src);
			
			// get the authInfo element preffix
			String apiPreffix = "";
			NamedNodeMap map = doc.getDocumentElement().getAttributes();
			for(int i=0; i<map.getLength(); i++) {
				Node node = map.item(i);
				if(node.getNodeValue().equalsIgnoreCase("urn:uddi-org:api_v3")) {
					String nodeXmlns = node.getNodeName();
					int colon = nodeXmlns.indexOf(":");
					apiPreffix = (colon != -1)? nodeXmlns.substring(colon + 1) : nodeXmlns; 
					break;
				}
			}
			
			Node node = doc.getElementsByTagName(apiPreffix + ":authInfo").item(0);
			node.setTextContent(authInfo);
			
			DOMImplementationRegistry registry = DOMImplementationRegistry
					.newInstance();
			DOMImplementationLS impl = (DOMImplementationLS) registry
					.getDOMImplementation("LS");
			LSSerializer writer = impl.createLSSerializer();
			String docAsString = writer.writeToString(doc);
	
			return docAsString;
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	private static String readReader(Reader reader) {
		Writer w = new StringWriter();
		try {
			char[] b = new char[4096];
			int size;
			while ((size = reader.read(b)) != -1) {
				w.write(b, 0, size);
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			if (w != null) {
				try {
					w.close();
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
		}
		return w.toString();
	}


	private static String readTestAsciiResource(String resourceName) {
		StringBuffer contentBuffer = new StringBuffer();
		URL url = Thread.currentThread().getContextClassLoader().getResource(resourceName);
		
		BufferedReader in = null;
		try {
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			String line = null;
			while((line = in.readLine()) != null) {
				contentBuffer.append(line + "\n");
			}
			
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
		}
		
		return contentBuffer.toString();
	}
	
}
