package org.jboss.soa.esb.samples.quickstart.juddi.tck;

import java.util.List;
import java.util.Map;

import org.apache.juddi.jaxb.EntityCreator;

import org.uddi.api_v3.BusinessService;
import org.uddi.api_v3.DeleteService;
import org.uddi.api_v3.GetServiceDetail;
import org.uddi.api_v3.SaveService;
import org.uddi.api_v3.ServiceDetail;

import org.uddi.v3_service.UDDIInquiryPortType;
import org.uddi.v3_service.UDDIPublicationPortType;

/**
 * 
 * @author <a href="mailto:kstam@apache.org">Kurt T Stam</a>
 * @author <a href="mailto:jfaath@apache.org">Jeff Faath</a>
 * @author <a href="mailto:mbaluch@redhat.com">Marek Baluch</a>
 */
public class TckBusinessService {

	public final static String NOTIFIER_SERVICE_XML = "org/jboss/soa/esb/qa/tests/failover/juddi/xml/subscriptionnotifier/businessService.xml";
	public final static String NOTIFIER_SERVICE_KEY = "uddi:uddi.joepublisher.com:notifierone";

	public final static String JOE_SERVICE_XML = "org/jboss/soa/esb/qa/tests/failover/juddi/xml/joepublisher/businessService.xml";
	public final static String JOE_SERVICE_KEY = "uddi:uddi.joepublisher.com:serviceone";
	public final static String SAM_SERVICE_XML = "org/jboss/soa/esb/qa/tests/failover/juddi/xml/samsyndicator/businessService.xml";
	public final static String SAM_SERVICE_KEY = "uddi:www.samco.com:listingservice";

	private UDDIPublicationPortType publication = null;
	private UDDIInquiryPortType inquiry = null;

	public TckBusinessService(UDDIPublicationPortType publication, UDDIInquiryPortType inquiry) {
		this.publication = publication;
		this.inquiry = inquiry;
	}

	public void saveServices(String authInfo, String serviceXML, String serviceKey, int numberOfCopies)
			throws Exception
	{
		try {
			Map<String, BusinessService> serviceInList = TckResourceLoader.copyBusinessServices(serviceXML, serviceKey, numberOfCopies);
			SaveService ss = new SaveService();
			for (String key : serviceInList.keySet()) {
				ss.setAuthInfo(authInfo);
				ss.getBusinessService().add(serviceInList.get(key));
			}
			publication.saveService(ss);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public void saveService(String authInfo, String serviceXML, String serviceKey) 
		throws Exception 
	{
		try {
			// First save the entity
			SaveService ss = new SaveService();
			ss.setAuthInfo(authInfo);
	
			BusinessService bsIn = (BusinessService) EntityCreator.buildFromDoc(serviceXML, "org.uddi.api_v3");
			ss.getBusinessService().add(bsIn);
			publication.saveService(ss);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	public void deleteService(String authInfo, String serviceKey)
			throws Exception 
	{
		try {
			// Delete the entity and make sure it is removed
			DeleteService ds = new DeleteService();
			ds.setAuthInfo(authInfo);
	
			ds.getServiceKey().add(serviceKey);
			publication.deleteService(ds);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

	public void deleteServices(String authInfo, String serviceKey,
			int numberOfCopies) throws Exception 
	{
		try {
			for (int i = 0; i < numberOfCopies; i++) {
				// Delete the entity and make sure it is removed
				DeleteService ds = new DeleteService();
				ds.setAuthInfo(authInfo);
	
				ds.getServiceKey().add(serviceKey + "-" + i);
				publication.deleteService(ds);
				System.out.println("Deleted Service with key " + serviceKey + "-" + i);
			}
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}

}