package org.jboss.soa.esb.samples.quickstart.juddi.tck;

import org.apache.juddi.v3.client.config.UDDIClerk;
import org.apache.juddi.v3.client.config.UDDIClerkManager;
import org.apache.juddi.v3.client.config.UDDIClientContainer;
import org.apache.juddi.v3.client.config.UDDINode;
import org.apache.juddi.v3.client.transport.Transport;

import org.uddi.api_v3.AuthToken;
import org.uddi.api_v3.DiscardAuthToken;
import org.uddi.api_v3.GetAuthToken;

import org.uddi.v3_service.UDDISecurityPortType;

import org.jboss.soa.esb.samples.quickstart.juddi.Utils;

/**
 * 
 * @author <a href="mailto:kstam@apache.org">Kurt T Stam</a>
 * @author <a href="mailto:jfaath@apache.org">Jeff Faath</a>
 * @author <a href="mailto:mbaluch@redhat.com">Marek Baluch</a>
 */
public class TckSecurity {

	public static String getAuthToken(UDDISecurityPortType securityService, String pubId, String cred) throws Exception {
		try {
			GetAuthToken ga = new GetAuthToken();
			ga.setUserID(pubId);
			ga.setCred(cred);
	
			AuthToken token = securityService.getAuthToken(ga);
			
			return token.getAuthInfo();
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public static void discardAuthToken(UDDISecurityPortType securityService, String authInfo) throws Exception {
		
		try {
			DiscardAuthToken dToken = new DiscardAuthToken();
			dToken.setAuthInfo(authInfo);
			
			securityService.discardAuthToken(dToken);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public static String getAuthToken(UDDIClerkManager manager, String clerkName) throws Exception {
		try {
			// make sure manager is started
			manager.start();
			
			UDDIClientContainer.addClerkManager(manager);
			
			UDDIClerk clerk = manager.getClientConfig().getUDDIClerks().get(clerkName);
			UDDINode node = clerk.getUDDINode();
			String clazz = node.getProxyTransport();
			Class<?> transportClass = Utils.forName(clazz, Transport.class);
			Transport transport = (Transport) transportClass.getConstructor(String.class).newInstance(node.getName());
			
			return getAuthToken(transport.getUDDISecurityService(), clerk.getPublisher(), clerk.getPassword());
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
	public static void discardAuthToken(UDDIClerkManager manager, String clerkName, String authInfo) throws Exception {
		try {
			// make sure manager is started
			manager.start();
			
			UDDIClientContainer.addClerkManager(manager);
			
			UDDIClerk clerk = manager.getClientConfig().getUDDIClerks().get(clerkName);
			UDDINode node = clerk.getUDDINode();
			String clazz = node.getProxyTransport();
			Class<?> transportClass = Utils.forName(clazz, Transport.class);
			Transport transport = (Transport) transportClass.getConstructor(String.class).newInstance(node.getName());
			
			discardAuthToken(transport.getUDDISecurityService(), authInfo);
		} catch (Exception e) {
			System.out.println("#### Exception: " + e);
			throw e;
		}
	}
	
}
